-- BACKEND_MEM.vhd
-- Revision 1, 04/20/2005
-- BACKEMD_MEM is a structure of memory instantiations to form the backend memory of Actel's Core1553BRT which can be accessed by  
--              a Core8051 as host processor.
-- BACKEND_MEM ports must be connected to the system as below
-- CLK: MIL-STD_1553 system clock
-- Ports which their name starts with 'CPU': Connect to the corresponding Core8051 memory interface ports. For example CPU_memdatao
--                                           connects to the memdatao port of Core8051
-- Ports which their name starts with 'RT': Connect to the corresponding Core1553BRT memory inteface ports. For example RT_MEMADDR
--                                          connects to the MEMADDR port of Core1553BRT
--
-- In this implementation, the MSB of MEMADDR port of Core1553BRT is unsed and can be left floating in the system.


library IEEE;
use IEEE.std_logic_1164.all;

entity BACKEND_MEM is port (
       CLK, CPU_memrd, CPU_memwr, RT_MEMWRn, RT_MEMRDn: in std_logic;
       CPU_memaddr: in std_logic_vector (10 downto 0);
       CPU_memdatao: in std_logic_vector (7 downto 0);
       RT_MEMDOUT: in std_logic_vector (15 downto 0);
       CPU_memdatai: out std_logic_vector (7 downto 0);
       -- MSB of 1553BRT MEMADDR (MEMADDR[10]) output is unused and can be left floating.
       RT_MEMADDR: in std_logic_vector (9 downto 0);
       RT_MEMDIN: out std_logic_vector (15 downto 0)
       );
end BACKEND_MEM;

architecture TOP of BACKEND_MEM is 

component TOP_MEM port(
        WDATA: in std_logic_vector (15 downto 0);
        RDATA: out std_logic_vector (15 downto 0);
        RCLK, WCLK,  RENB: in std_logic;
        WADDR, RADDR: in std_logic_vector (9 downto 0);
        WENB: in std_logic_vector (1 downto 0));
end component;

component BOTTOM_MEM port(
        WDATA: in std_logic_vector (15 downto 0);
        RDATA: out std_logic_vector (15 downto 0);
        RCLK, WCLK, WENB, RENB: in std_logic;
        WADDR, RADDR: in std_logic_vector (9 downto 0));
end component;

signal bottom_read: std_logic_vector ( 15 downto 0);
signal top_wen: std_logic_vector ( 1 downto 0);

begin

TOP_HALF: TOP_MEM port map(
          WDATA => (CPU_memdatao & CPU_memdatao),
          RDATA => RT_MEMDIN,
          RCLK => CLK,
          WCLk => CLk,
          RENB => RT_MEMRDn,
          WENB => top_wen,
          WADDR => CPU_memaddr(10 downto 1),
          RADDR => RT_MEMADDR
          );

BOTTOM_HALF: BOTTOM_MEM port map(
          WDATA => RT_MEMDOUT,
          RDATA => bottom_read,
          RCLK => CLK,
          WCLk => CLK,
          RENB => not CPU_memrd,
          WENB => RT_MEMWRn,
          WADDR => RT_MEMADDR,
          RADDR => CPU_memaddr(10 downto 1)
          );
process (bottom_read, CPU_memaddr(0))
begin
if (CPU_memaddr(0) = '0') then 
CPU_memdatai <= bottom_read(7 downto 0);
else 
CPU_memdatai <= bottom_read(15 downto 8);
end if;
end process; 

--memory enables active-low and cpu enables active_hi
top_wen(1) <= not(CPU_memwr and CPU_memaddr(0)); 
top_wen(0) <= not(not(CPU_memwr) and CPU_memaddr(0));  

end TOP;
